@@ -7,6 +7,8 @@ project_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
 if project_path not in sys.path:
     sys.path.insert(0, project_path)
 
+import hashlib
+import time
 from flask import Flask, request, url_for, g, render_template
 from flask_wtf.csrf import CsrfProtect
 from flask.ext.uploads import configure_uploads
@@ -52,12 +54,7 @@ def create_app():
     register_jinja(app)
     register_error_handle(app)
     register_uploadsets(app)
-    register_admin(app)
-
-    # before every request
-    @app.before_request
-    def before_request():
-        g.user = get_current_user()
+    register_hooks(app)
 
     return app
 
@@ -67,6 +64,9 @@ def register_jinja(app):
     from jinja2 import Markup
     from .utils import filters
 
+    if not hasattr(app, '_static_hash'):
+        app._static_hash = {}
+
     app.jinja_env.filters['timesince'] = filters.timesince
 
     # inject vars into template context
@@ -87,8 +87,27 @@ def register_jinja(app):
         return url_for(request.endpoint, **combined_args)
 
     def static(filename):
-        """静态资源url"""
-        return url_for('static', filename=filename)
+        """静态资源url
+
+        计算资源内容hash作为query string，并缓存起来。
+        """
+        if app.testing:
+            return url_for('static', filename=filename)
+
+        if filename in app._static_hash:
+            return app._static_hash[filename]
+
+        path = os.path.join(app.static_folder, filename)
+        if not os.path.exists(path):
+            return url_for('static', filename=filename)
+
+        with open(path, 'r') as f:
+            content = f.read()
+            hash = hashlib.md5(content).hexdigest()
+
+        url = '%s?v=%s' % (url_for('static', filename=filename), hash[:10])
+        app._static_hash[filename] = url
+        return url
 
     def script(path):
         """script标签"""
@@ -159,14 +178,19 @@ def register_uploadsets(app):
     configure_uploads(app, (avatars))
 
 
-def register_admin(app):
-    """注册Flask-Admin"""
-    from flask.ext.admin import Admin
-    from flask.ext.admin.contrib.sqla import ModelView
-    from .models import db, User
+def register_hooks(app):
+    @app.before_request
+    def before_request():
+        g.user = get_current_user()
+        if g.user and g.user.is_admin:
+            g._before_request_time = time.time()
 
-    admin = Admin(app)
-    admin.add_view(ModelView(User, db.session))
+    @app.after_request
+    def after_request(response):
+        if hasattr(g, '_before_request_time'):
+            delta = time.time() - g._before_request_time
+            response.headers['X-Render-Time'] = delta * 1000
+        return response
 
 
 def _get_template_name(template_reference):
